#weather-page {
    text-shadow: inset 1px 1px 3px black;
    icon-shadow: inset 1px 1px 3px black;
}

#weather-page-content-view {
    background-color: black;
    background-size: cover;
    background-position: center;
    background-origin: border-box;
    background-clip: border-box;
}

#weather-page-content-view.weather-clear {
    background-image: url("weather-clear.jpg");
}

#weather-page-content-view.weather-clear-night {
    background-image: url("weather-clear-night.jpg");
}

#weather-page-content-view.weather-few-clouds {
    background-image: url("weather-few-clouds.jpg");
}

#weather-page-content-view.weather-few-clouds-night {
    background-image: url("weather-few-clouds-night.jpg");
}

#weather-page-content-view.weather-fog {
    background-image: url("weather-fog.jpg");
    background-position: center 20%;
}

#weather-page-content-view.weather-overcast {
    background-image: url("weather-overcast.jpg");
}

#weather-page-content-view.weather-showers,
#weather-page-content-view.weather-showers-scattered {
    background-image: url("weather-showers.jpg");
}

#weather-page-content-view.weather-snow {
    background-image: url("weather-snow.jpg");
}

#weather-page-content-view.weather-storm {
    background-image: url("weather-storm.jpg");
    background-position: center 80%;
}

#loading-label {
    padding-top: 24px;
    font-size: 1.5em;
}

#temperature-label {
    font-size: 4em;
}
#condition-label {
    font-size: 2.5em;
    padding-bottom: 12px; /* keep in sync with conditions-image */
}
#attribution-label {
    font-size: small;
}

#conditions-grid *:backdrop {
    color: @theme_fg_color;
}

#conditions-image {
    padding: 12px;
}

.content-view.cell {
    font-weight: bold;
}